local addonName = ...

local string_gsub = string.gsub
local string_len = string.len
local string_match = string.match
local string_sub = string.sub
local string_trim = string.trim

local BNGetNumFriends = BNGetNumFriends
local CHAT_BN_WHISPER_SEND = CHAT_BN_WHISPER_SEND
local ChatTypeInfo = ChatTypeInfo

local _, id, alias, find, replace

local aliases = {}
local sunames = {}

-- backup these original API for later usage
local oBNGetFriendInfo = BNGetFriendInfo
local oBNGetFriendInfoByID = BNGetFriendInfoByID
local oBNTokenFindName = BNTokenFindName

-- returns what the function below returns, to save us from having to retype the same code
function BNGetFriendInfo(index, ...)
  id = oBNGetFriendInfo(index, ...)
  if index <= BNGetNumFriends() then
    return BNGetFriendInfoByID(id)
  end
end

-- alter returned values and use alias from note if possible (prefix with hidden RealID tag for other parts of the UI)
function BNGetFriendInfoByID(presenceID)
  local presenceID, presenceName, battleTag, isBattleTagPresence, toonName, toonID, client, isOnline, lastOnline, isAFK, isDND, messageText, noteText, isRIDFriend, broadcastTime, canSoR = oBNGetFriendInfoByID(presenceID)
  if noteText then
    alias = string_match(noteText, "%^(.-)%$")
    if alias and alias ~= "" then
      --noteText = string_trim(string_gsub(noteText, "%^"..string_gsub(alias, "[%.%-%+%(%)%[%]%?%^%$]", "%%%1").."%$", ""))
      aliases[alias] = presenceName
      sunames[presenceName] = alias
      if isBattleTagPresence then
        battleTag = alias
      end
      alias = "|Kf0|k|k"..alias -- a lot of API check first part for "|K" so we supply one, this indicates that this is indeed a RealID friend and not a in-game character name!
      presenceName = alias
    end
  end
  return presenceID, presenceName, battleTag, isBattleTagPresence, toonName, toonID, client, isOnline, lastOnline, isAFK, isDND, messageText, noteText, isRIDFriend, broadcastTime, canSoR
end

-- extract alias from message then return empty message (I am not Superman, it's an internal API anyway, not too easy to override!)
function BNTokenFindName(target, ...)
  if "|Kf0|k|k" == string_sub(target, 1, 8) then
    alias = string_trim(string_sub(target, 9))
    return aliases[alias] or alias, ""
  end
  return oBNTokenFindName(target, ...)
end

-- change the header to show the alias instead of the surname
hooksecurefunc("ChatEdit_UpdateHeader", function(editBox)
  local type = editBox:GetAttribute("chatType")
  if not type then
    return
  end
  local info = ChatTypeInfo[type]
  local header = _G[editBox:GetName().."Header"]
  local headerSuffix = _G[editBox:GetName().."HeaderSuffix"]
  if not header then
    return
  end
  if type == "BN_WHISPER" then
    header:SetWidth(0)
    alias = editBox:GetAttribute("tellTarget")
    alias = sunames[alias] or alias
    header:SetFormattedText(CHAT_BN_WHISPER_SEND, alias)
    local headerWidth = header:GetRight() - header:GetLeft()
    local editBoxWidth = editBox:GetRight() - editBox:GetLeft()
    if headerWidth > editBoxWidth / 2 then
      header:SetWidth(editBoxWidth / 2)
      headerSuffix:Show()
    else
      headerSuffix:Hide()
    end
    header:SetTextColor(info.r, info.g, info.b)
    headerSuffix:SetTextColor(info.r, info.g, info.b)
    editBox:SetTextInsets(15 + header:GetWidth() + (headerSuffix:IsShown() and headerSuffix:GetWidth() or 0), 13, 0, 0)
    editBox:SetTextColor(info.r, info.g, info.b)
    editBox.focusLeft:SetVertexColor(info.r, info.g, info.b)
    editBox.focusRight:SetVertexColor(info.r, info.g, info.b)
    editBox.focusMid:SetVertexColor(info.r, info.g, info.b)
  end
end)

-- scan all existing chat frames
local ChatFrames = {}
for i = 1, NUM_CHAT_WINDOWS do
  table.insert(ChatFrames, "ChatFrame"..i)
end

-- backup original AddMessage widget API
local AddMessage = DEFAULT_CHAT_FRAME.AddMessage

-- custom way to replace RealID name with alias
local function Custom_AddMessage(self, text, ...)
  id, alias = text:match("|Kf(%d+)|k(.-)|k")
  if id and alias then
    find = "|Kf"..id.."|k"..alias.."|k|h|Kf"..id.."|k"..alias.."|k|h"
    replace = sunames["|Kf"..id.."|k"..alias.."|k"]
    text = string_gsub(text, find, replace and ("|h"..replace.."|h") or find)
  end
  AddMessage(self, text, ...)
end

-- register ourselves with each chat frame and it's own AddMessage
for _, chatName in ipairs(ChatFrames) do
  local chatFrame = _G[chatName]
  if chatFrame then
    chatFrame.AddMessage = Custom_AddMessage
  end
end
